# -*- coding: utf-8 -*-

import base64
import grpc
import json
import os
import pathlib
import shutil
import tempfile

from concurrent import futures
from lintaosp.flow.flow_pb2 import FlowReply
from lintaosp.flow.flow_pb2_grpc import (
    add_FlowProtoServicer_to_server,
    FlowProtoServicer,
)

LINT_NAME = "lintaosp"
MAX_WORKERS = 10


class FlowException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Flow(object):
    def __init__(self, config):
        if config is None:
            raise FlowException("config invalid")
        self._config = config

    def _serve(self, routine):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=MAX_WORKERS))
        add_FlowProtoServicer_to_server(FlowProto(routine), server)
        server.add_insecure_port(self._config.listen_url)
        server.start()
        server.wait_for_termination()

    def run(self, routine):
        self._serve(routine)


class FlowProto(FlowProtoServicer):
    def __init__(self, routine):
        self._routine = routine

    def _build(self, data):
        def _helper(root, dirs, file, data):
            pathlib.Path(os.path.join(root, dirs)).mkdir(parents=True, exist_ok=True)
            p = pathlib.Path(os.path.join(root, dirs, file))
            with p.open("w") as f:
                f.write(base64.b64decode(data).decode("utf-8"))

        if len(data) == 0:
            return None
        buf = json.loads(data)
        root = tempfile.mkdtemp(prefix=LINT_NAME + "-")
        for key, val in buf.items():
            _helper(root, os.path.dirname(key), os.path.basename(key), val)

        return root

    def _clean(self, project):
        if os.path.exists(project):
            shutil.rmtree(project)

    def SendFlow(self, request, _):
        if len(request.message) == 0:
            return FlowReply(message="")
        project = self._build(request.message.strip())
        if project is None or not os.path.exists(project):
            return FlowReply(message="")
        buf = self._routine(project)
        self._clean(project)
        return FlowReply(message=json.dumps({LINT_NAME: buf}))



# -*- coding: utf-8 -*-

import sys

from lintaosp.aosp.aosp import Aosp, AospException
from lintaosp.cmd.argument import Argument
from lintaosp.cmd.banner import BANNER
from lintaosp.config.config import Config, ConfigException
from lintaosp.flow.flow import Flow, FlowException
from lintaosp.logger.logger import Logger
from lintaosp.queue.queue import Queue, QueueException


def main():
    print(BANNER)

    argument = Argument()
    arg = argument.parse(sys.argv)

    try:
        config = Config()
        config.config_file = arg.config_file
        config.lint_project = arg.lint_project
        config.listen_url = arg.listen_url
        config.output_file = arg.output_file
    except ConfigException as e:
        Logger.error(str(e))
        return -1

    try:
        aosp = Aosp(config)
    except AospException as e:
        Logger.error(str(e))
        return -2

    Logger.info("lint running")

    if len(config.listen_url) != 0:
        try:
            flow = Flow(config)
            flow.run(aosp.routine)
        except FlowException as e:
            Logger.error(str(e))
            return -3
    else:
        try:
            queue = Queue(config)
            queue.run(aosp.routine, config.lint_project)
        except QueueException as e:
            Logger.error(str(e))
            return -4

    Logger.info("lint exiting")

    return 0



# -*- coding: utf-8 -*-

import os
import yaml

INPUT_TEXT_SEP = ","


class ConfigFile:
    APIVERSION = "apiVersion"
    KIND = "kind"
    METADATA = "metadata"
    SPEC = "spec"


class MetaData:
    NAME = "name"


class ConfigException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Config(object):
    def __init__(self):
        self._config_file = None
        self._lint_project = ""
        self._listen_url = ""
        self._output_file = ""

    @property
    def config_file(self):
        return self._config_file

    @config_file.setter
    def config_file(self, name):
        if not isinstance(name, str) or len(name.strip()) == 0:
            raise ConfigException("name invalid")
        if not name.endswith(".yml") and not name.endswith(".yaml"):
            raise ConfigException("suffix invalid")
        if not os.path.exists(name):
            raise ConfigException("%s not found" % name)
        with open(name) as file:
            self._config_file = yaml.load(file, Loader=yaml.FullLoader)
        if self._config_file is None:
            raise ConfigException("config invalid")

    @property
    def lint_project(self):
        return self._lint_project

    @lint_project.setter
    def lint_project(self, name):
        if not isinstance(name, str) or not os.path.exists(name):
            raise ConfigException("name invalid")
        self._lint_project = name.strip()

    @property
    def listen_url(self):
        return self._listen_url

    @listen_url.setter
    def listen_url(self, url):
        if not isinstance(url, str):
            raise ConfigException("url invalid")
        self._listen_url = url.strip()

    @property
    def output_file(self):
        return self._output_file

    @output_file.setter
    def output_file(self, name):
        if not isinstance(name, str):
            raise ConfigException("name invalid")
        if len(name.strip()) != 0:
            if (
                not name.endswith(".json")
                and not name.endswith(".txt")
                and not name.endswith(".xlsx")
            ):
                raise ConfigException("suffix invalid")
            if os.path.exists(name):
                raise ConfigException("%s already exist" % name)
        self._output_file = name.strip()



# -*- coding: utf-8 -*-

import subprocess
import sys

from lintaosp.aosp.abstract import AospAbstract
from lintaosp.proto.proto import Format, Type

CHECK_SEP = ","

LINT_LEN_MIN = 3
LINT_SEP = ":"
LINT_TYPE = [Type.ERROR, Type.INFO, Type.WARN]


class SdkException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Sdk(AospAbstract):
    def __init__(self, config):
        if config is None:
            raise SdkException("config invalid")
        super().__init__(config)

    def _execution(self, project):
        self._remove(project, "build.gradle")
        return self._lint(project)

    def _parse(self, data):
        buf = []
        for item in data.splitlines():
            b = item.strip().split(LINT_SEP)
            if len(b) < LINT_LEN_MIN:
                continue
            file = b[0].strip()
            if b[1].strip() in LINT_TYPE:
                line = 0
                _type = b[1].strip()
                details = LINT_SEP.join(b[2:]).strip()
            elif type(int(b[1].strip())) == int:
                line = int(b[1].strip())
                _type = b[2].strip() if b[2].strip() in LINT_TYPE else ""
                details = LINT_SEP.join(b[3:]).strip()
            else:
                continue
            buf.append(
                {
                    Format.FILE: file,
                    Format.LINE: line,
                    Format.TYPE: _type,
                    Format.DETAILS: details,
                }
            )
        return buf

    def _popen(self, cmd, stdin=None):
        return subprocess.Popen(
            cmd, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

    def _lint(self, project):
        tool = "lint.bat" if "win" in sys.platform else "lint"
        check = CHECK_SEP.join(self._config)
        cmd = [
            tool,
            "--check",
            check,
            "--disable",
            "LintError",
            "--nolines",
            "--quiet",
            project,
        ]
        with self._popen(cmd) as proc:
            out, err = proc.communicate()
            if proc.returncode != 0:
                raise SdkException(err.strip().decode("utf-8"))
        return self._parse(out.strip().decode("utf-8"))

    def _remove(self, project, file):
        cmd = ["find", project, "-type", "f", "-name", file, "-exec", "rm -f {} \\;"]
        with self._popen(cmd) as proc:
            _, _ = proc.communicate()
            if proc.returncode != 0:
                pass



# -*- coding: utf-8 -*-

import colorama
import sys
import time


class Logger(object):
    def __init__(self):
        pass

    @staticmethod
    def debug(msg):
        sys.stderr.write(
            u"{debug}{time} DEBUG:{reset} {msg}\n".format(
                debug=colorama.Fore.GREEN + colorama.Style.BRIGHT,
                time=time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())),
                reset=colorama.Style.RESET_ALL,
                msg=msg,
            )
        )

    @staticmethod
    def error(msg):
        sys.stderr.write(
            u"{error}{time} ERROR:{reset} {msg}\n".format(
                error=colorama.Fore.RED + colorama.Style.BRIGHT,
                time=time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())),
                reset=colorama.Style.RESET_ALL,
                msg=msg,
            )
        )

    @staticmethod
    def info(msg):
        sys.stderr.write(
            u"{info}{time} INFO:{reset} {msg}\n".format(
                info=colorama.Fore.WHITE + colorama.Style.BRIGHT,
                time=time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())),
                reset=colorama.Style.RESET_ALL,
                msg=msg,
            )
        )

    @staticmethod
    def warn(msg):
        sys.stderr.write(
            u"{warn}{time} WARN:{reset} {msg}\n".format(
                warn=colorama.Fore.YELLOW + colorama.Style.BRIGHT,
                time=time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())),
                reset=colorama.Style.RESET_ALL,
                msg=msg,
            )
        )



# -*- coding: utf-8 -*-


"""Prototype
{
  "lintaosp": [
    {
      "file": "name",
      "line": 1,
      "type": "Error",
      "details": "text"
    }
  ]
}
"""


class Format:
    DETAILS = "details"
    FILE = "file"
    LINE = "line"
    TYPE = "type"


class Type:
    ERROR = "Error"
    INFO = "Info"
    WARN = "Warning"



__author__ = "Jia Jia"
__author_email__ = "angersax@sina.com"
__description__ = "Lint AOSP"
__license__ = "Apache-2.0"
__title__ = "lintaosp"
__url__ = "https://github.com/craftslab/lintaosp"
__version__ = "1.2.0"
